==> netbeans_8.0.2+dfsg1.orig.tar.xz: netbeans-8.0.2/parsing.api/src/org/netbeans/modules/parsing/impl/Schedulers.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 * 
 * Contributor(s):
 * 
 * Portions Copyrighted 2008 Sun Microsystems, Inc.
 */

package org.netbeans.modules.parsing.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import org.netbeans.modules.parsing.spi.Scheduler;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.WeakListeners;


/**
 *
 * @author Jan Jancura
 */
public class Schedulers {

    //@GuardedBy("Schedulers.class")
    private static Collection<? extends Scheduler> taskSchedulers;
    //@GuardedBy("Scheduler.class")
    private static Lookup.Result<Scheduler> result;
    //@GuardedBy("Scheduler.class")
    private static LookupListener listener;
    
    static synchronized void init () {
        if (taskSchedulers == null) {
            if (result == null) {
                assert listener == null;
                listener = new LkpListener();
                result = Lookup.getDefault().lookupResult(Scheduler.class);
                result.addLookupListener(WeakListeners.create(
                    LookupListener.class,
                    listener,
                    result));
            }
            taskSchedulers = Collections.unmodifiableCollection(
                    new ArrayList<Scheduler>(result.allInstances()));
        }
    }

    /**
     * For tests only.
     */
    static synchronized Collection<? extends Scheduler> getSchedulers () {
        init();
        return taskSchedulers;
    }

    private static class LkpListener implements LookupListener {
        @Override
        public void resultChanged(LookupEvent ev) {
            synchronized (Schedulers.class) {
                taskSchedulers = null;
            }
        }
    }
}

==> netbeans_8.1+dfsg2.orig.tar.xz: netbeans-8.1/parsing.api/src/org/netbeans/modules/parsing/impl/Schedulers.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 * 
 * Contributor(s):
 * 
 * Portions Copyrighted 2008 Sun Microsystems, Inc.
 */

package org.netbeans.modules.parsing.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import org.netbeans.modules.parsing.spi.Scheduler;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.WeakListeners;


/**
 *
 * @author Jan Jancura
 */
public class Schedulers {

    //@GuardedBy("Schedulers.class")
    private Collection<? extends Scheduler> taskSchedulers;
    //@GuardedBy("Scheduler.class")
    private Lookup.Result<Scheduler> result;
    //@GuardedBy("Scheduler.class")
    private LookupListener listener;
    
    private Lookup lookup;
    
    private static final Schedulers INSTANCE = new Schedulers(null);
    
    private Schedulers(Lookup lookup) {
        this.lookup = lookup;
    }
    
    synchronized void init () {
        if (taskSchedulers == null) {
            if (result == null) {
                assert listener == null;
                listener = new LkpListener();
                result = (lookup == null ?Lookup.getDefault() : lookup).lookupResult(Scheduler.class);
                result.addLookupListener(WeakListeners.create(
                    LookupListener.class,
                    listener,
                    result));
            }
            taskSchedulers = Collections.unmodifiableCollection(
                    new ArrayList<Scheduler>(result.allInstances()));
        }
    }
    
    public static Schedulers getInstance(Lookup lkp) {
        return new Schedulers(lkp);
    }
    
    public static Collection<? extends Scheduler> getSchedulers () {
        Schedulers inst = INSTANCE;
        return inst.getSchedulerList();
    }

    /**
     * For tests only.
     */
    public Collection<? extends Scheduler> getSchedulerList() {
        init();
        return taskSchedulers;
    }

    private class LkpListener implements LookupListener {
        @Override
        public void resultChanged(LookupEvent ev) {
            synchronized (Schedulers.class) {
                taskSchedulers = null;
            }
        }
    }
}
